@@ -1118,9 +1118,9 @@ gegl_buffer_try_lock (GeglBuffer *buffer)
     GeglTileBackend *backend = gegl_buffer_backend (buffer);
 
     gboolean threaded = gegl_config_threads ()>1;
-    
+
     if (threaded)
-      g_mutex_lock (&buffer->tile_storage->mutex);
+      g_rec_mutex_lock (&buffer->tile_storage->mutex);
     if (buffer->lock_count > 0)
       buffer->lock_count++;
     else if (gegl_tile_backend_file_try_lock (GEGL_TILE_BACKEND_FILE (backend)))
@@ -1128,7 +1128,7 @@ gegl_buffer_try_lock (GeglBuffer *buffer)
     else
       ret = FALSE;
     if (threaded)
-      g_mutex_unlock (&buffer->tile_storage->mutex);
+      g_rec_mutex_unlock (&buffer->tile_storage->mutex);
   }
 
   return ret;
@@ -1157,7 +1157,7 @@ gegl_buffer_unlock (GeglBuffer *buffer)
     GeglTileBackend *backend = gegl_buffer_backend (buffer);
 
     if (threaded)
-      g_mutex_lock (&buffer->tile_storage->mutex);
+      g_rec_mutex_lock (&buffer->tile_storage->mutex);
 
     buffer->lock_count--;
     g_assert (buffer->lock_count >= 0);
@@ -1166,7 +1166,7 @@ gegl_buffer_unlock (GeglBuffer *buffer)
       ret = gegl_tile_backend_file_unlock (GEGL_TILE_BACKEND_FILE (backend));
 
     if (threaded)
-      g_mutex_unlock (&buffer->tile_storage->mutex);
+      g_rec_mutex_unlock (&buffer->tile_storage->mutex);
   }
 
   return ret;
@@ -1197,7 +1197,7 @@ glong gegl_buffer_signal_connect (GeglBuffer *buffer,
 {
   buffer->changed_signal_connections++;
   return g_signal_connect(buffer, detailed_signal, c_handler, data);
-} 
+}
 
 GeglTile *
 gegl_buffer_get_tile (GeglBuffer *buffer,
@@ -1208,16 +1208,9 @@ gegl_buffer_get_tile (GeglBuffer *buffer,
   GeglTileSource  *source  = (GeglTileSource*)buffer;
   GeglTile *tile;
 
-  static int threaded = -1;
-
-  if (threaded == -1)
-  {
-    threaded = gegl_config_threads () > 1;
-  }
-
   g_assert (source);
 
-  if (threaded)
+  if (gegl_config_threads () > 1)
   {
     GeglTileStorage *tile_storage = buffer->tile_storage;
     g_assert (tile_storage);
