@@ -246,6 +246,7 @@ gboolean          gegl_buffer_set_abyss      (GeglBuffer          *buffer,
  * data is converted to the desired BablFormat, if the BablFormat stored and
  * fetched is the same this amounts to a series of memcpy's aligned to demux
  * the tile structure into a linear buffer.
+ *
  */
 void            gegl_buffer_get               (GeglBuffer          *buffer,
                                                const GeglRectangle *rect,
@@ -259,9 +260,9 @@ void            gegl_buffer_get               (GeglBuffer          *buffer,
  * gegl_buffer_set: (skip)
  * @buffer: the buffer to modify.
  * @rect: the coordinates we want to change the data of and the width/height of
- * the linear buffer being set, scale specifies the scaling factor applied to
+ * the linear buffer being set.
  * the data when setting.
- * @scale_level: the scale level being set, 0 = 1:1 = default = base mipmap level,
+ * @mipmap_level: the scale level being set, 0 = 1:1 = default = base mipmap level,
  * 1 = 1:2, 2=1:4, 3=1:8 ..
  * @format: the babl_format the linear buffer @src.
  * @src: linear buffer of image data to be stored in @buffer.
@@ -272,7 +273,7 @@ void            gegl_buffer_get               (GeglBuffer          *buffer,
  */
 void            gegl_buffer_set               (GeglBuffer          *buffer,
                                                const GeglRectangle *rect,
-                                               gint                 scale_level,
+                                               gint                 mipmap_level,
                                                const Babl          *format,
                                                const void          *src,
                                                gint                 rowstride);
@@ -315,7 +316,7 @@ void            gegl_buffer_set_pattern       (GeglBuffer          *buffer,
  *
  * Get the babl format of the buffer, this might not be the format the buffer
  * was originally created with, you need to use gegl_buffer_set_format (buf,
- * NULL); to retireve the original format (potentially having save away the
+ * NULL); to retrieve the original format (potentially having saved away the
  * original format of the buffer to re-set it.)
  *
  * Returns: the babl format used for storing pixels in the buffer.
@@ -334,7 +335,7 @@ const Babl *    gegl_buffer_get_format        (GeglBuffer           *buffer);
  * original format. If NULL is passed in the format of the buffer is reset to
  * the original format.
  *
- * Returns: the new babl format or NULL if it the passed in buffer was
+ * Returns: the new babl format or NULL if the passed-in buffer was
  * incompatible (then the original format is still used).
  */
 const Babl *    gegl_buffer_set_format        (GeglBuffer          *buffer,
@@ -346,7 +347,7 @@ const Babl *    gegl_buffer_set_format        (GeglBuffer          *buffer,
  * @roi: a rectangular region
  *
  * Clears the provided rectangular region by setting all the associated memory
- * to 0
+ * to 0.
  */
 void            gegl_buffer_clear             (GeglBuffer          *buffer,
                                                const GeglRectangle *roi);
@@ -361,7 +362,7 @@ void            gegl_buffer_clear             (GeglBuffer          *buffer,
  * @dst_rect: position of upper left destination pixel, or NULL for top
  * left coordinates of the buffer extents.
  *
- * copies a region from source buffer to destination buffer.
+ * Copy a region from source buffer to destination buffer.
  *
  * If the babl_formats of the buffers are the same, and the tile boundaries
  * align, this will create copy-on-write tiles in the destination buffer.
