@@ -219,24 +219,26 @@ gegl_tile_handler_cache_command (GeglTileSource  *tile_store,
     {
       case GEGL_TILE_FLUSH:
         {
-          GList     *link;
-
           if (gegl_cl_is_accelerated ())
             gegl_buffer_cl_cache_flush2 (cache, NULL);
 
           if (cache->count)
             {
-              for (link = g_queue_peek_head_link (cache_queue); link; link = link->next)
-                {
-                  CacheItem *item = LINK_GET_ITEM (link);
-                  GeglTile  *tile = item->tile;
+              CacheItem      *item;
+              GHashTableIter  iter;
+              gpointer        key, value;
 
-                  if (tile != NULL &&
-                      item->handler == cache)
-                    {
-                      gegl_tile_store (tile);
-                    }
+              g_mutex_lock (&mutex);
+
+              g_hash_table_iter_init (&iter, cache->items);
+              while (g_hash_table_iter_next (&iter, &key, &value))
+                {
+                  item = (CacheItem *) value;
+                  if (item->tile)
+                    gegl_tile_store (item->tile);
                 }
+
+              g_mutex_unlock (&mutex);
             }
         }
         break;
@@ -293,20 +295,34 @@ gegl_tile_handler_cache_wash (GeglTileHandlerCache *cache)
   gint       wash_tiles = cache_wash_percentage * length / 100;
   GList     *link;
 
-  for (link = g_queue_peek_head_link (cache_queue); link; link = link->next)
+  g_mutex_lock (&mutex);
+
+  for (link = g_queue_peek_tail_link (cache_queue);
+       link && count < wash_tiles;
+       link = link->prev, count++)
     {
       CacheItem *item = LINK_GET_ITEM (link);
       GeglTile  *tile = item->tile;
 
-      count++;
       if (!gegl_tile_is_stored (tile))
-        if (count > length - wash_tiles)
+        {
           last_dirty = tile;
+          if (last_dirty->tile_storage)
+            g_object_ref (last_dirty->tile_storage);
+          gegl_tile_ref (last_dirty);
+
+          break;
+        }
     }
 
+  g_mutex_unlock (&mutex);
+
   if (last_dirty != NULL)
     {
       gegl_tile_store (last_dirty);
+      gegl_tile_unref (last_dirty);
+      if (last_dirty->tile_storage)
+        g_object_unref (last_dirty->tile_storage);
       return TRUE;
     }
   return FALSE;
@@ -345,16 +361,21 @@ gegl_tile_handler_cache_get_tile (GeglTileHandlerCache *cache,
   result = cache_lookup (cache, x, y, z);
   if (result)
     {
+      GeglTile *volatile tile;
+
       g_queue_unlink (cache_queue, &result->link);
       g_queue_push_head_link (cache_queue, &result->link);
-      g_mutex_unlock (&mutex);
       while (result->tile == NULL)
       {
         g_printerr ("NULL tile in %s %p %i %i %i %p\n", __FUNCTION__, result, result->x, result->y, result->z,
                 result->tile);
+        g_mutex_unlock (&mutex);
         return NULL;
       }
-      return gegl_tile_ref (result->tile);
+      gegl_tile_ref (result->tile);
+      tile = result->tile;
+      g_mutex_unlock (&mutex);
+      return tile;
     }
   g_mutex_unlock (&mutex);
   return NULL;
@@ -384,8 +405,10 @@ drop_hot_tile (GeglTile *tile)
 
   if (storage)
     {
+#if 0 /* the storage's mutex should already be locked at this point */
       if (gegl_config_threads()>1)
         g_rec_mutex_lock (&storage->mutex);
+#endif
 
       if (storage->hot_tile == tile)
         {
@@ -393,8 +416,10 @@ drop_hot_tile (GeglTile *tile)
           storage->hot_tile = NULL;
         }
 
+#if 0
       if (gegl_config_threads()>1)
         g_rec_mutex_unlock (&storage->mutex);
+#endif
     }
 }
 
@@ -403,22 +428,76 @@ gegl_tile_handler_cache_trim (GeglTileHandlerCache *cache)
 {
   GList *link;
 
-  link = g_queue_pop_tail_link (cache_queue);
+  link = g_queue_peek_tail_link (cache_queue);
 
-  if (link != NULL)
+  while (cache_total > gegl_config()->tile_cache_size)
     {
-      CacheItem *last_writable = LINK_GET_ITEM (link);
-      GeglTile *tile = last_writable->tile;
+      CacheItem       *last_writable;
+      GeglTile        *tile;
+      GeglTileStorage *storage;
+      gboolean         dirty;
+      GList           *prev_link;
 
+#ifdef GEGL_DEBUG_CACHE_HITS
+      GEGL_NOTE(GEGL_DEBUG_CACHE, "cache_total:"G_GUINT64_FORMAT" > cache_size:"G_GUINT64_FORMAT, cache_total, gegl_config()->tile_cache_size);
+      GEGL_NOTE(GEGL_DEBUG_CACHE, "%f%% hit:%i miss:%i  %i]", cache_hits*100.0/(cache_hits+cache_misses), cache_hits, cache_misses, g_queue_get_length (cache_queue));
+#endif
+
+      for (; link != NULL; link = g_list_previous (link))
+        {
+          last_writable = LINK_GET_ITEM (link);
+          tile          = last_writable->tile;
+
+          /* if the tile's ref-count is greater than one, then someone is still
+           * using the tile, and we must keep it in the cache, so that we can
+           * return the same tile object upon request; otherwise, we would end
+           * up with two different tile objects referring to the same tile.
+           */
+          if (tile->ref_count > 1)
+            continue;
+
+          storage = tile->tile_storage;
+
+          dirty = storage && ! gegl_tile_is_stored (tile);
+          if (dirty && gegl_config_threads()>1)
+            {
+              /* XXX:  if the tile is dirty, then gegl_tile_unref() will try to
+               * store it, acquiring the storage mutex in the process.  this
+               * can lead to a deadlock if another thread is already holding
+               * that mutex, and is waiting on the global cache mutex, or on a
+               * tile storage mutex held by the current thread.  try locking
+               * the storage mutex here, and skip the tile if it fails.
+               */
+              if (! g_rec_mutex_trylock (&storage->mutex))
+                continue;
+            }
+
+          break;
+        }
+
+      if (link == NULL)
+        return FALSE;
+
+      prev_link = g_list_previous (link);
+      g_queue_unlink (cache_queue, link);
       g_hash_table_remove (last_writable->handler->items, last_writable);
       cache_total -= tile->size;
-      drop_hot_tile (tile);
+      last_writable->handler->count--;
+      /* drop_hot_tile (tile); */ /* XXX:  no use in trying to drop the hot
+                                   * tile, since this tile can't be it --
+                                   * the hot tile will have a ref-count of
+                                   * at least two.
+                                   */
       gegl_tile_unref (tile);
+
+      if (dirty && gegl_config_threads()>1)
+        g_rec_mutex_unlock (&storage->mutex);
+
       g_slice_free (CacheItem, last_writable);
-      return TRUE;
+      link = prev_link;
     }
 
-  return FALSE;
+  return TRUE;
 }
 
 static void
@@ -434,17 +513,22 @@ gegl_tile_handler_cache_invalidate (GeglTileHandlerCache *cache,
   if (item)
     {
       cache_total -= item->tile->size;
+      cache->count--;
+
+      g_queue_unlink (cache_queue, &item->link);
+      g_hash_table_remove (cache->items, item);
+
+      g_mutex_unlock (&mutex);
+
       drop_hot_tile (item->tile);
       item->tile->tile_storage = NULL;
       gegl_tile_mark_as_stored (item->tile); /* to cheat it out of being stored */
       gegl_tile_unref (item->tile);
 
-      g_queue_unlink (cache_queue, &item->link);
-      g_hash_table_remove (cache->items, item);
-
       g_slice_free (CacheItem, item);
     }
-  g_mutex_unlock (&mutex);
+  else
+    g_mutex_unlock (&mutex);
 }
 
 
@@ -513,14 +597,8 @@ gegl_tile_handler_cache_insert (GeglTileHandlerCache *cache,
 
   g_hash_table_insert (cache->items, item, item);
 
-  while (cache_total > gegl_config()->tile_cache_size)
-    {
-#ifdef GEGL_DEBUG_CACHE_HITS
-      GEGL_NOTE(GEGL_DEBUG_CACHE, "cache_total:"G_GUINT64_FORMAT" > cache_size:"G_GUINT64_FORMAT, cache_total, gegl_config()->tile_cache_size);
-      GEGL_NOTE(GEGL_DEBUG_CACHE, "%f%% hit:%i miss:%i  %i]", cache_hits*100.0/(cache_hits+cache_misses), cache_hits, cache_misses, g_queue_get_length (cache_queue));
-#endif
-      gegl_tile_handler_cache_trim (cache);
-    }
+  gegl_tile_handler_cache_trim (cache);
+
   g_mutex_unlock (&mutex);
 }
 
