@@ -46,6 +46,8 @@ remove_in_betweens (GeglNode *nop_raw,
       int count = gegl_node_get_consumers (iter, "output", &nodes, NULL);
       if (count)
         iter = nodes[0];
+      else
+        iter = NULL;
       g_free (nodes);
       if (iter && iter != nop_transformed)
         collect = g_list_append (collect, iter);
@@ -57,6 +59,28 @@ remove_in_betweens (GeglNode *nop_raw,
     }
 }
 
+static void each_knot (const GeglPathItem *path_node,
+                       gpointer user_data)
+{
+  GString *str = user_data;
+  gchar fstr[G_ASCII_DTOSTR_BUF_SIZE];
+  g_ascii_dtostr (fstr, sizeof(fstr), path_node->point[0].x);
+  g_string_append_printf (str, " %s=", fstr);
+  g_ascii_dtostr (fstr, sizeof(fstr), path_node->point[0].y);
+  g_string_append_printf (str, "%s ", fstr);
+}
+
+static void each_knot_rel (const GeglPathItem *path_node,
+                           gpointer user_data)
+{
+  GString *str = user_data;
+  gchar fstr[G_ASCII_DTOSTR_BUF_SIZE];
+  g_ascii_dtostr (fstr, sizeof(fstr), path_node->point[0].x);
+  g_string_append_printf (str, " %s=", fstr);
+  g_ascii_dtostr (fstr, sizeof(fstr), path_node->point[0].y);
+  g_string_append_printf (str, "%srel ", fstr);
+}
+
 void
 gegl_create_chain_argv (char      **ops,
                         GeglNode   *start,
@@ -99,8 +123,17 @@ gegl_create_chain_argv (char      **ops,
               value[-1] = '\0';
               if (strstr (value, "rel"))
                 {
+                  char tmpbuf[1024];
+                  GQuark rel_quark;
                   gegl_path_append (path, 'L', g_strtod (key, NULL),
                                     make_rel (value));
+
+                  sprintf (tmpbuf, "%s-rel", prop);
+                  rel_quark = g_quark_from_string (tmpbuf);
+                  g_object_set_qdata_full (G_OBJECT (
+                                           new),
+                                           rel_quark,
+                                           g_strdup (value), g_free);
                 }
               else
                 {
@@ -119,12 +152,38 @@ gegl_create_chain_argv (char      **ops,
           if (strchr (*arg, '}'))
             {
               gdouble y = 0;
+              char tmpbuf[1024];
+              GQuark anim_quark;
+              sprintf (tmpbuf, "%s-anim", prop);
+              anim_quark = g_quark_from_string (tmpbuf);
+
               if (time == 0.0) /* avoiding ugly start interpolation artifact */
-                time = 0.4;
+                time = 0.001;
               gegl_path_calc_y_for_x (g_object_get_qdata (G_OBJECT (new),
-                                                          g_quark_from_string(
-                                                            prop)), time, &y);
-              gegl_node_set (new, prop, y, NULL);
+                                      anim_quark), time, &y);
+
+              {
+                GParamSpec *pspec = gegl_operation_find_property (gegl_node_get_operation (new), prop);
+
+                if (GEGL_IS_PARAM_SPEC_DOUBLE (pspec))
+                {
+                  GParamSpecDouble *double_spec = (void*)pspec;
+                  if (y <= double_spec->minimum)
+                    y = double_spec->minimum;
+                  if (y >= double_spec->maximum)
+                    y = double_spec->maximum;
+                  gegl_node_set (new, prop, y, NULL);
+                }
+                else if (GEGL_IS_PARAM_SPEC_INT (pspec))
+                {
+                  GParamSpecInt *int_spec = (void*)pspec;
+                  if (y <= int_spec->minimum)
+                    y = int_spec->minimum;
+                  if (y >= int_spec->maximum)
+                    y = int_spec->maximum;
+                  gegl_node_set (new, prop, (int)(y), NULL);
+                }
+              }
 
               in_keyframes = 0;
             }
@@ -247,6 +306,10 @@ gegl_create_chain_argv (char      **ops,
                           }
                         else
                           {
+                            char tmpbuf[1024];
+                            GQuark anim_quark;
+                            sprintf (tmpbuf, "%s-anim", key);
+                            anim_quark = g_quark_from_string (tmpbuf);
                             path = gegl_path_new ();
                             in_keyframes = 1;
                             if (prop)
@@ -254,8 +317,7 @@ gegl_create_chain_argv (char      **ops,
                             prop = g_strdup (key);
 
                             g_object_set_qdata_full (G_OBJECT (
-                                                       new), g_quark_from_string(
-                                                       key), path,
+                                                       new), anim_quark, path,
                                                      g_object_unref);
                           }
 
@@ -366,10 +428,15 @@ gegl_create_chain_argv (char      **ops,
                       {
                         if (strstr (value, "rel"))
                           {
+                            char tmpbuf[1024];
+                            GQuark rel_quark;
+                            sprintf (tmpbuf, "%s-rel", key);
+                            rel_quark = g_quark_from_string (tmpbuf);
+
                             g_object_set_qdata_full (G_OBJECT (
                                                        new),
-                                                     g_quark_from_string(
-                                                       key), g_strdup (
+                                                     rel_quark,
+                                                       g_strdup (
                                                        value), g_free);
 
                             if (g_type_is_a (target_type, G_TYPE_INT))
@@ -538,7 +605,7 @@ gegl_create_chain_argv (char      **ops,
                                                proxy), "operation",
                                              level_op[level], NULL);
 
-                  if (iter[level])
+                  if (iter[level] && gegl_node_has_pad (new, "input"))
                     gegl_node_link_many (iter[level], new, proxy, NULL);
                   else
                     gegl_node_link_many (new, proxy, NULL);
@@ -629,6 +696,7 @@ gegl_create_chain (const char *str, GeglNode *op_start, GeglNode *op_end,
     }
 }
 
+/* TODO: serialize keyframed properties */
 static gchar *
 gegl_serialize2 (GeglNode *start, GeglNode *end, const char *basepath,
                  GHashTable *ht, GeglSerializeFlag flags)
@@ -721,22 +789,41 @@ gegl_serialize2 (GeglNode *start, GeglNode *end, const char *basepath,
                 GType property_type = G_PARAM_SPEC_VALUE_TYPE (properties[i]);
                 const GValue*default_value = g_param_spec_get_default_value (
                   properties[i]);
+                char tmpbuf[1024];
+                GeglPath *path = NULL;
                 gboolean printed = FALSE;
-
+                char *rel_orig = NULL;
+                GQuark anim_quark, rel_quark;
+                sprintf (tmpbuf, "%s-anim", properties[i]->name);
+                anim_quark = g_quark_from_string (tmpbuf);
+                sprintf (tmpbuf, "%s-rel", properties[i]->name);
+                rel_quark = g_quark_from_string (tmpbuf);
+                path = g_object_get_qdata (G_OBJECT (iter), anim_quark);
+                rel_orig = g_object_get_qdata (G_OBJECT (iter), rel_quark);
 
                 if (property_type == G_TYPE_FLOAT)
                   {
                     gfloat defval = g_value_get_float (default_value);
                     gfloat value;
-                    gchar str[G_ASCII_DTOSTR_BUF_SIZE];
                     gegl_node_get (iter, properties[i]->name, &value, NULL);
-                    if (value != defval || (!trim_defaults))
+
+                    if (path)
+                    {
+                      g_string_append_printf (s2, " %s={ ", property_name);
+                      if (rel_orig)
+                        gegl_path_foreach (path, each_knot_rel, s2);
+                      else
+                        gegl_path_foreach (path, each_knot, s2);
+                      g_string_append_printf (s2, " } ");
+                    }
+                    else if (value != defval || (!trim_defaults))
                       {
+                        gchar str[G_ASCII_DTOSTR_BUF_SIZE];
                         g_ascii_dtostr (str, sizeof(str), value);
                         if (flags & GEGL_SERIALIZE_INDENT)
                           g_string_append_printf (s2, "  ");
-                        g_string_append_printf (s2, " %s=%s", property_name,
-                                                str);
+                        g_string_append_printf (s2, " %s=%s%s", property_name,
+                                                str, rel_orig?"rel":"");
                         printed = TRUE;
                       }
                   }
@@ -744,15 +831,25 @@ gegl_serialize2 (GeglNode *start, GeglNode *end, const char *basepath,
                   {
                     gdouble defval = g_value_get_double (default_value);
                     gdouble value;
-                    gchar str[G_ASCII_DTOSTR_BUF_SIZE];
                     gegl_node_get (iter, property_name, &value, NULL);
-                    if (value != defval || (!trim_defaults))
+
+                    if (path)
+                    {
+                      g_string_append_printf (s2, " %s={ ", property_name);
+                      if (rel_orig)
+                        gegl_path_foreach (path, each_knot_rel, s2);
+                      else
+                        gegl_path_foreach (path, each_knot, s2);
+                      g_string_append_printf (s2, " } ");
+                    }
+                    else if (value != defval || (!trim_defaults))
                       {
+                        gchar str[G_ASCII_DTOSTR_BUF_SIZE];
                         if (flags & GEGL_SERIALIZE_INDENT)
                           g_string_append_printf (s2, "  ");
                         g_ascii_dtostr (str, sizeof(str), value);
-                        g_string_append_printf (s2, " %s=%s", property_name,
-                                                str);
+                        g_string_append_printf (s2, " %s=%s%s", property_name,
+                                                str, rel_orig?"rel":"");
                         printed = TRUE;
                       }
                   }
@@ -762,13 +859,23 @@ gegl_serialize2 (GeglNode *start, GeglNode *end, const char *basepath,
                     gint value;
                     gchar str[64];
                     gegl_node_get (iter, properties[i]->name, &value, NULL);
-                    if (value != defval || (!trim_defaults))
+
+                    if (path)
+                    {
+                      g_string_append_printf (s2, " %s={ ", property_name);
+                      if (rel_orig)
+                        gegl_path_foreach (path, each_knot_rel, s2);
+                      else
+                        gegl_path_foreach (path, each_knot, s2);
+                      g_string_append_printf (s2, " } ");
+                    }
+                    else if (value != defval || (!trim_defaults))
                       {
                         if (flags & GEGL_SERIALIZE_INDENT)
                           g_string_append_printf (s2, "  ");
                         g_snprintf (str, sizeof (str), "%i", value);
-                        g_string_append_printf (s2, " %s=%s", property_name,
-                                                str);
+                        g_string_append_printf (s2, " %s=%s%s", property_name,
+                                                str, rel_orig?"rel":"");
                         printed = TRUE;
                       }
                   }
@@ -893,7 +1000,8 @@ gegl_serialize2 (GeglNode *start, GeglNode *end, const char *basepath,
                     {
                       char *str = gegl_serialize2 (NULL, aux, basepath, ht,
                                                    flags);
-                      g_string_append_printf (s2, " aux=[ %s ]\n", str);
+                      g_string_append_printf (s2, " aux=[ %s ]%s", str,
+                            (flags& GEGL_SERIALIZE_INDENT)?"\n":" ");
                       g_free (str);
                     }
                 }
