@@ -73,6 +73,7 @@ void gegl_op_matting_global_register_type(GTypeModule *module);
 void gegl_op_maze_register_type(GTypeModule *module);
 void gegl_op_mblur_register_type(GTypeModule *module);
 void gegl_op_mirrors_register_type(GTypeModule *module);
+void gegl_op_mix_register_type(GTypeModule *module);
 void gegl_op_mono_mixer_register_type(GTypeModule *module);
 void gegl_op_mosaic_register_type(GTypeModule *module);
 void gegl_op_motion_blur_circular_register_type(GTypeModule *module);
@@ -116,6 +117,7 @@ void gegl_op_sinus_register_type(GTypeModule *module);
 void gegl_op_slic_register_type(GTypeModule *module);
 void gegl_op_snn_mean_register_type(GTypeModule *module);
 void gegl_op_softglow_register_type(GTypeModule *module);
+void gegl_op_spiral_register_type(GTypeModule *module);
 void gegl_op_stress_register_type(GTypeModule *module);
 void gegl_op_stretch_contrast_hsv_register_type(GTypeModule *module);
 void gegl_op_stretch_contrast_register_type(GTypeModule *module);
@@ -234,6 +236,7 @@ gegl_module_register (GTypeModule *module)
   gegl_op_maze_register_type(module);
   gegl_op_mblur_register_type(module);
   gegl_op_mirrors_register_type(module);
+  gegl_op_mix_register_type(module);
   gegl_op_mono_mixer_register_type(module);
   gegl_op_mosaic_register_type(module);
   gegl_op_motion_blur_circular_register_type(module);
@@ -277,6 +280,7 @@ gegl_module_register (GTypeModule *module)
   gegl_op_slic_register_type(module);
   gegl_op_snn_mean_register_type(module);
   gegl_op_softglow_register_type(module);
+  gegl_op_spiral_register_type(module);
   gegl_op_stress_register_type(module);
   gegl_op_stretch_contrast_hsv_register_type(module);
   gegl_op_stretch_contrast_register_type(module);
