@@ -97,7 +97,11 @@ static const char* op_c_source =
 "  result.width  = o->width;                                                   \n"
 "  result.height = o->height;                                                  \n"
 "                                                                              \n"
-"  gegl_rectangle_intersect (&result, &result, in_rect);                       \n"
+"  /* in_rect sometimes ends up as 0,0,0,0 and in those cases - other          \n"
+"   * code ends up seg-faulting                                                \n"
+"   */                                                                         \n"
+"  if (in_rect->width != 0 && in_rect->height != 0)                            \n"
+"    gegl_rectangle_intersect (&result, &result, in_rect);                     \n"
 "                                                                              \n"
 "  return result;                                                              \n"
 "}                                                                             \n"
@@ -219,6 +223,7 @@ static const char* op_c_source =
 "      \"categories\",  \"core\",                                              \n"
 "      \"title\",       _(\"Crop\"),                                           \n"
 "      \"description\", _(\"Crop a buffer\"),                                  \n"
+"      \"reference-hash\", \"6f9f160434a4e9484d334c29122e5682\",               \n"
 "      \"reference-composition\", composition,                                 \n"
 "      NULL);                                                                  \n"
 "                                                                              \n"
