@@ -531,7 +531,7 @@ gegl_op_class_init (GeglOpClass *klass)
   gegl_operation_class_set_keys (operation_class,
     "name",           "gegl:path",
     "title",          _("Render Path"),
-    "categories",     "render",
+    "categories",     "render:vector",
     "reference-hash", "eba13d16fba3284ceee264073ebfa356",
     "reference-composition", composition,
     "description" , _("Renders a brush stroke"),
