@@ -1,5 +1,7 @@
 static const char* op_c_source =
-"/* XXX: moved to workshop due to bug #777699, excessive memory allocations    \n"
+"/* XXX: moved to workshop due to bug #777699, excessive memory allocations -  \n"
+" *      the op is not a crucial image processing tool - and can possibly even \n"
+" *      be fully removed.                                                     \n"
 " */                                                                           \n"
 "                                                                              \n"
 "                                                                              \n"
