@@ -223,8 +223,8 @@ static const char* op_c_source =
 "    \"<gegl>\"                                                                \n"
 "    \"<node operation='gegl:gradient-map'>\"                                  \n"
 "    \"  <params>\"                                                            \n"
-"    \"    <param name='color1'>#x410404</param>\"                             \n"
-"    \"    <param name='color2'>#x22FFFA</param>\"                             \n"
+"    \"    <param name='color1'>#410404</param>\"                              \n"
+"    \"    <param name='color2'>#22FFFA</param>\"                              \n"
 "    \"  </params>\"                                                           \n"
 "    \"</node>\"                                                               \n"
 "    \"<node operation='gegl:load'>\"                                          \n"
